/*---------------------------------------------------------------------------*\
  ==  == ====== ====   ====    |
                    \\     ||  | Multiphase Code Repository by HZDR
  ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
  ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
  ==  == ====== ====   ==  ==  |
-------------------------------------------------------------------------------
License
    This file is part of the Multiphase Code Repository by HZDR.

    Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
    Website: https://hzdr.de

    Multiphase Code Repository by HZDR is based on the free software for
    computational fluid dynamics (CFD) from the OpenFOAM Foundation.
    Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org

    If you are interested in which files are original OpenFOAM Foundation
    files, which OpenFOAM Foundation files were modified, and which files were
    newly created, see FILES.md.

    Multiphase Code Repository by HZDR is free software: you can redistribute
    it and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    Multiphase Code Repository by HZDR is distributed in the hope that it will
    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
    Public License for more details.

    You should have received a copy of the GNU General Public License along
    with the Multiphase Code Repository by HZDR. If not, see
    <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "cipsaMultiphaseEuler.H"
#include "fvmSup.H"

// * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * * //

void Foam::solvers::cipsaMultiphaseEuler::updateDmdtUEqn()
{
    forAll(movingPhases, movingPhasei)
    {
        phaseModel& phase = movingPhases_[movingPhasei];
        const volScalarField& alpha = phase;
        const volVectorField& UAlpha = phase.U();
        volScalarField& rhoAlpha = phase.rho();

        const PtrList<volScalarField> dmdts = fluid.dmdts();

        const dimensionedScalar ZERO(dimensionSet(1, -3, -1, 0, 0), Zero);

        dmdtUEqns.set
        (
            phase.index(),
            (
                fvm::Sp(ZERO, UAlpha)
              + fvm::Su(Zero, UAlpha)
            )
        );

        volScalarField source
        (
            volScalarField::New
            (
                IOobject::groupName("source", phase.name()),
                mesh,
                dimensionedScalar(dimDensity/dimTime, 0)
            )
        );

        if (fvModels().addsSupToField(rhoAlpha.name()))
        {
            source += fvModels().source(alpha, rhoAlpha)&rhoAlpha;
        }

        if (dmdts.set(phase.index()))
        {
            source += dmdts[phase.index()];
        }

        dmdtUEqns[phase.index()] += fvm::SuSp(source, UAlpha);
    }

    fillFields("dmdtPhiCoeff", dimensionSet(1, 0, -2, 0, 0), dmdtPhiCoeffs);

    // Add mass sources
    // Inspired by MomentumTransferPhaseSystem::addDmdtUfs() but with
    // fvm::SuSp() instead of fvm::Sp()
    {
        autoPtr<phaseSystem::dmdtfTable> dmdtfsPtr(fluid.dmdtfs());

        forAllConstIter(phaseSystem::dmdtfTable, dmdtfsPtr(), dmdtfIter)
        {
            const phaseInterface interface(fluid, dmdtfIter.key());

            const volScalarField& dmdtf(*dmdtfIter());
            const volScalarField dmdtf21(posPart(dmdtf));
            const volScalarField dmdtf12(negPart(dmdtf));

            phaseModel& phase1 = fluid_.phases()[interface.phase1().name()];
            phaseModel& phase2 = fluid_.phases()[interface.phase2().name()];

            if (!phase1.stationary())
            {
                dmdtUEqns[phase1.index()] -=
                    pos(phase1 - phase1.residualAlpha())*dmdtf21*phase2.U()
                  + fvm::SuSp(dmdtf12, phase1.URef());

                dmdtPhiCoeffs[phase1.index()] +=
                    pos(alphafs[phase1.index()] - phase1.residualAlpha())
                   *fvc::interpolate(dmdtf21)*MRF.absolute(phase2.phi());
            }

            if (!phase2.stationary())
            {
                dmdtUEqns[phase2.index()] +=
                    pos(phase2 - phase2.residualAlpha())*dmdtf12*phase1.U()
                  - fvm::SuSp(-dmdtf21, phase2.URef());

                dmdtPhiCoeffs[phase2.index()] -=
                    pos(alphafs[phase2.index()] - phase2.residualAlpha())
                   *fvc::interpolate(dmdtf12)*MRF.absolute(phase1.phi());
            }
        }
    }
}


// ************************************************************************* //
