# Test Case for Phase Mass Controller fvModel

## Description

This test case features a single valve in a 2D column tray similar to the one
investigated by Wiedemann et al.[^1]. Initially a mixture of continuous liquid
and bubbles (20 % volume fraction) is resting at the bottom of the domain.
Subsequently, continuous gas is injected via a volumetric source close to the
bottom of the domain. During the process bubbles and droplets are generated,
which are eventually being absorbed in the according continuous phase. The
continuous gas is leaving the domain via the top boundary. Nevertheless, a small
fraction of droplets reaches the top boundary, where it leaves the domain. This
slow drainage of liquid can be observed for longer run times.

The total mass of the physical phases might not be perfectly conserved in the
process of morphology transfers, i.e. during phase transfer from a continuous
to a disperse phase or vice-versa. This is a consequence of the segregated
solution procedure. The fvModel phaseMassController is therefore applied to
prevent that these inaccuracies accumulate over time.

## Technical Information

- continuous phases: liquidC, gasC
- disperse phase: liquidD, gasD
- phase transfer:
  - from liquidC to liquidD (droplet generation)
  - from liquidD to liquidC (droplet deposition)
  - from gasC to gasD (bubble generation)
  - from gasD to gasC (bubble bursting)
- 2D
- 15x35 cells
- fvModel 'degassing' for reducing droplets
- fvModel 'phaseMassController' for controlling total amount of liquid mass

## Pre-Processing

The cellZone for the injector volume source is created with topoSet. The phases
are initialised with setFields.

## Post-Processing

Both liquid and both gas phases are summed up resulting the fields
alpha.liquidTotal and alpha.gasTotal.

The Gnuplot script `createGraphs` is located in the `validation/` directory and
it plots the evolution of individual liquid and gas volumes as well as the total
liquid volume over time.

Total volume of all three phases is stored via the volFieldValue functionObject.

## Keywords

2D, multiphase, transient, water, air, isothermal, compressible,
phaseTransfer, buoyancy, droplets, bubbles

## Literature

[^1]: Wiedemann, P., Meller, R., Schubert, M., & Hampel, U. (2023).
      Application of a hybrid multiphase CFD approach to the simulation of
      gas–liquid flow at a trapezoid fixed valve for distillation trays.
      Chemical Engineering Research and Design, 193, 777–786.
