# Test Case for Resolved Interface Drag

## Description

This test is designed for testing against the analytical solution for a
stratified laminar flow. The fluid on the top ($`\mu_2`$) and the fluid at the
bottom ($`\mu_1`$) are both driven by a constant volume force with same
strength but opposite direction. Bottom and top wall allow slip.

Assume that the interface between both fluids is located at $`y=0`$ and that
there acts a volume force according to:

```math
q =
\left\{\begin{array}{lr}
        - K & \text{for} &  0 < y \le h \\
        + K & \text{for} & -h \le y < 0
\end{array}\right\}
```

The analytical solution for laminar fully developed flow is:

```math
u - U_\mathrm{I} =
\left\{\begin{array}{lr}
        + \frac{K}{\mu_2} \left( \frac{1}{2} y^2 - hy \right) & \text{for} &  0 \le y \le h \\
        - \frac{K}{\mu_1} \left( \frac{1}{2} y^2 + hy \right) & \text{for} & -h \le y \le 0
\end{array}\right\}
```

The interface velocity $`U_\mathrm{I}`$ is arbitrary and mainly results from the
given initial condition.

## Technical Information

- horizontally cyclic flow
- top and bottom: slip walls
- volume fraction is fixed in time
- domain consists of a single vertical row of cells
- no gravitational acceleration

## Pre-Processing

Volume fraction is set by setFields with a value of 0.5 for the centre cell.

## Post-Processing

GraphCell is used to record cell values. The analytical solution is plotted as a
reference. The final solution is shifted to the origin.

## Keywords

cyclic, isothermal, incompressible, analytical-solution
