# Test Case for Interface Parallel Buoyancy

## Description

Test case to demonstrate and verify the functionality of the
*interfaceParallelBuoyancy* method (switch in PIMPLE dictionary). The case
features a periodic channel consisting of a single row of cells. The channel is
tilted by 60 Degree and it is bounded by free-slip walls. Air and water are
distributed such that they share a smeared interface, which is parallel to the
walls. Due to the *disperse* contribution of the buoyancy force, both phases do
accelerate in different directions: water in gravitational direction and air in
the opposite direction. It is demonstrated, that *interfaceParallelBuoyancy*
eliminates the interface normal contribution of this force, while preserving the
interface parallel part.

## Technical Information

* 2D with a single row of cells
* boundaries:
  * top and bottom: free slip conditions
  * left and right: cyclic
* alpha fields frozen (nAlphaCorr 0)
* channel tiled 60 degree to the right (by rotation of vector of gravity)
* interfaceParallelBuoyancy
* weak drag coupling between phases: StrubeljTiselj (tau 1e1)
* run for single time step only, as there is no wall friction and ,hence, fluid
  velocity is indefinite

## Pre-Processing

* setFields: initialise smeared interface
* m4 script: calculation of vector of gravity for specific tilt angle

## Post-Processing

FunctionObjects used at run time:

* singleGraph (p, p_rgh, alpha.air, U.water, U.air)

Bash script:

* Gnuplot script createGraphs

Results are compared to reference data. Find all result files in the validation
directory.

## Keywords

multimorph, stratified flow, buoyancy, interface parallel
