#!/bin/bash
#---------------------------------*- sh -*-------------------------------------
# ==  == ====== ====   ====    |
#                   \\     ||  | Multiphase Code Repository by HZDR
# ======   //   ||  || ===//   | Website: https://doi.org/10.14278/rodare.767
# ||  ||  //    ||  // || \\   | License: GPL-3.0-or-later
# ==  == ====== ====   ==  ==  |
#------------------------------------------------------------------------------
# License
#     This file is part of the Multiphase Code Repository by HZDR.
#
#     Copyright (C) 2024 by Helmholtz-Zentrum Dresden-Rossendorf e.V. (HZDR),
#     Website: https://hzdr.de
#
#     Multiphase Code Repository by HZDR is based on the free software for
#     computational fluid dynamics (CFD) from the OpenFOAM Foundation.
#     Copyright (C) 2024 by OpenFOAM Foundation, Website: https://openfoam.org
#
#     If you are interested in which files are original OpenFOAM Foundation
#     files, which OpenFOAM Foundation files were modified, and which files were
#     newly created, see FILES.md.
#
#     Multiphase Code Repository by HZDR is free software: you can redistribute
#     it and/or modify it under the terms of the GNU General Public License as
#     published by the Free Software Foundation, either version 3 of the
#     License, or (at your option) any later version.
#
#     Multiphase Code Repository by HZDR is distributed in the hope that it will
#     be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
#     Public License for more details.
#
#     You should have received a copy of the GNU General Public License along
#     with the Multiphase Code Repository by HZDR. If not, see
#     <http://www.gnu.org/licenses/>.
#
# Description
#     Script to check the environment variables for banner check in hooks
#
#------------------------------------------------------------------------------

GREPEXE="grep -q -e"
PCREGREPEXE="pcregrep -q -M"
ERROR=0

# shellcheck source=../../../bin/tools/HookFunctions
. "${WM_PROJECT_SITE}/${WM_PROJECT_VERSION}/bin/tools/HookFunctions"

# Check that header variables are present
if [ -z "$sourceBanner" ]
then
    echo "Error: sourceBanner variable not set correctly"
    ((++ERROR))
fi

if [ -z "$scriptBanner" ]
then
    echo "Error: scriptBanner variable not set"
    ((++ERROR))
fi

if [ -z "$dictBanner" ]
then
    echo "Error: dictBanner variable not set"
    ((++ERROR))
fi

# Check for correct source code banners
files=("hzdr/correct/app.C")
for file in "${files[@]}"
do
    echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    echo "Checking correct $file"

    if ! $GREPEXE "$sourceRegExp" "$file";
    then
        echo "    Error: Hook will not detect file."
        ((++ERROR))
    else
        echo "    Hook will detect file."
    fi

    if ! $PCREGREPEXE "$sourceBanner" "$file";
    then
        echo "    Error: Banner check failed."
        ((++ERROR))
    else
        echo "    Banner check successful."
    fi
done

# Check for wrong source code banners
files=("./hzdr/wrong/app.C")
for file in "${files[@]}"
do
    echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    echo "Checking wrong $file"


    if ! $GREPEXE "$sourceRegExp" "$file";
    then
        echo "    Error: Hook will not detect file."
        ((++ERROR))
    else
        echo "    Hook will detect file."
    fi

    if ! $PCREGREPEXE "$sourceBanner" "$file";
    then
        echo "    Banner check failed successfully."
    else
        echo "    Error: Banner check did not failed as expected."
        ((++ERROR))
    fi
done

# Check for correct script banners
files=("./hzdr/correct/bash.sh" "./cases/correct/bash.sh")
for file in "${files[@]}"
do
    echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    echo "Checking correct $file"

    if ! $GREPEXE "$sourceRegExp" "$file";
    then
        echo "    Error: Hook will not detect file."
        ((++ERROR))
    else
        echo "    Hook will detect file."
    fi

    if ! $PCREGREPEXE "$scriptBanner" "$file";
    then
        echo "    Error: Banner check failed."
        ((++ERROR))
    else
        echo "    Banner check successful."
    fi
done

# Check for wrong script banners
files=("./hzdr/wrong/bash.sh" "./cases/wrong/bash.sh")
for file in "${files[@]}"
do
    echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    echo "Checking wrong $file"

    if ! $GREPEXE "$sourceRegExp" "$file";
    then
        echo "    Error: Hook will not detect file."
        ((++ERROR))
    else
        echo "    Hook will detect file."
    fi

    if ! $PCREGREPEXE "$scriptBanner" "$file";
    then
        echo "    Banner check failed successfully."
    else
        echo "    Error: Banner check did not failed as expected."
        ((++ERROR))
    fi
done

# Check for correct dict banners
files=( "./hzdr/correct/controlDict" "./cases/correct/controlDict")
for file in "${files[@]}"
do
    echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    echo "Checking correct $file"


    if ! $GREPEXE "$dictRegExp" "$file"
    then
        echo "    Error: Hook will not detect file."
        ((++ERROR))
    else
        echo "    Hook will detect file."
    fi

    if ! $PCREGREPEXE "$dictBanner" "$file";
    then
        echo "    Error: Banner check failed."
        ((++ERROR))
    else
        echo "    Banner check successful."
    fi
done

# Check for wrong dict banners
files=("./hzdr/wrong/controlDict" "./cases/wrong/controlDict")
for file in "${files[@]}"
do
    echo "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"
    echo "Checking wrong $file"

    if ! $GREPEXE "$dictRegExp" "$file";
    then
        echo "    Error: Hook will not detect file."
        ((++ERROR))
    else
        echo "    Hook will detect file."
    fi

    if ! $PCREGREPEXE "$dictBanner" "$file";
    then
        echo "    Banner check failed successfully."
    else
        echo "    Error: Banner check did not failed as expected."
        ((++ERROR))
    fi
done

if [ "$ERROR" -eq 0 ]
then
    exit 0
else
    exit 1
fi
